import { lazy, object, optional, Schema, string } from '../schema';
import { Address, addressSchema } from './address';

/**
 * Defines the fields that are included in the request body of a request
 * to the `CreateCustomerCard` endpoint.
 */
export interface CreateCustomerCardRequest {
  /**
   * A card nonce representing the credit card to link to the customer.
   * Card nonces are generated by the Square payment form when customers enter
   * their card information. For more information, see
   * [Walkthrough: Integrate Square Payments in a Website](https://developer.squareup.com/docs/web-payments/take-card-payment).
   * __NOTE:__ Card nonces generated by digital wallets (such as Apple Pay)
   * cannot be used to create a customer card.
   */
  cardNonce: string;
  /**
   * Represents a postal address in a country.
   * For more information, see [Working with Addresses](https://developer.squareup.com/docs/build-basics/working-with-addresses).
   */
  billingAddress?: Address;
  /** The full name printed on the credit card. */
  cardholderName?: string;
  /**
   * An identifying token generated by [Payments.verifyBuyer()](https://developer.squareup.com/reference/sdks/web/payments/objects/Payments#Payments.verifyBuyer).
   * Verification tokens encapsulate customer device information and 3-D Secure
   * challenge results to indicate that Square has verified the buyer identity.
   */
  verificationToken?: string;
}

export const createCustomerCardRequestSchema: Schema<CreateCustomerCardRequest> = object(
  {
    cardNonce: ['card_nonce', string()],
    billingAddress: ['billing_address', optional(lazy(() => addressSchema))],
    cardholderName: ['cardholder_name', optional(string())],
    verificationToken: ['verification_token', optional(string())],
  }
);
